@@ -6,7 +6,7 @@ import copy
 from CifFile import CifFile, ReadCif, StarFile, CifBlock
 from typing import List, Tuple, Union, Any
 
-DEBUG = True
+DEBUG = False
 
 
 def pretty(d, indent=0, print_values=True):
@@ -55,7 +55,7 @@ def blockname_lookupdict_from_blockid(cif: Union[dict, CifFile]) -> dict:
 
 def get_blockname_from_block_id(lookup_dict: dict, block_id: str) -> str:
     """
-    Pycifrw makes datablock names all lower case. When I make up a pd+block_id from a dataname, sometimes the user has a
+    Pycifrw makes datablock names all lower case. When I make up a pd_block_id from a dataname, sometimes the user has a
     case-dependent reference somewhere in their CIF, and so I can't find the datablock. This looks in my lookup dictionary
     two ways to see if it can be found.
     :param lookup_dict: the dictionary mapping block_id to datablock
@@ -105,7 +105,7 @@ def get_from_cif(cif: Union[dict, CifFile], itemname: str, default: Any = None):
 
 def get_hkld_ids(structure: Union[dict, CifFile]) -> dict:
     """
-    Given a cif block representing aa structure, get the h,k,l, and d values of the reflections
+    Given a cif block representing a structure, get the h,k,l, and d values of the reflections
     and return them in a dictionary. or returns None if '_refln_d_spacing' doesn't exist
 
     :param structure: a cif dictionary or PyCIFRW
@@ -118,7 +118,7 @@ def get_hkld_ids(structure: Union[dict, CifFile]) -> dict:
     id_local = get_from_cif(structure, '_pd_refln_phase_id')
 
     if d_local is None:
-        return None
+        return {}
     d_len = len(d_local)
     # h,k,l, and d must be of the same length
     if h_local is None or len(h_local) != d_len:
@@ -166,8 +166,8 @@ def get_hkld_from_matching_id(hkld_ids: dict) -> dict:
     :param hkld_ids: dictionary of h,k,l,d,id values
     :return: a list containing the corresponding hkld values, or None, if none match.
     """
-    if hkld_ids is None:
-        return None
+    if not hkld_ids:
+        return {}
     unique_ids = set(hkld_ids["id"])
     return {idv: {"h": [h for h, idval in zip(hkld_ids["h"], hkld_ids["id"]) if idval == idv],
                   "k": [k for k, idval in zip(hkld_ids["k"], hkld_ids["id"]) if idval == idv],
@@ -196,7 +196,7 @@ def _split_val_err(ve: str, default_error: str = "zero") -> Tuple[float, Union[f
         if ve in {".", "?"}:
             val = float("nan")
             err = float("nan")
-            return (val, err)
+            return val, err
         else:
             val = float(ve)
         if default_error == "sqrt":
@@ -205,7 +205,7 @@ def _split_val_err(ve: str, default_error: str = "zero") -> Tuple[float, Union[f
             err = 0.0
         else:
             err = None
-        return (val, err)
+        return val, err
 
     val, err = re.search("([0-9.]+)\(([0-9]*)\)", ve).groups()
     if "." not in val:  # it makes it much easier if there is a decimal place to count
@@ -235,9 +235,9 @@ def split_val_err(value: Union[str, List[str]], default_error: str = "none") ->
         errs.append(e)
 
     if isList:
-        return (vals, errs)
+        return vals, errs
     else:
-        return (vals[0], errs[0])
+        return vals[0], errs[0]
 
 
 def calc_cumchi2(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_dataname_err: str = None, ymod_dataname: str = None):
@@ -245,7 +245,11 @@ def calc_cumchi2(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_dat
     Calculate the cumulative chi-squared statistic using the given yobs, ycalc, uncertinaty, and y_modifier.
     This is an indication of the value of chi2 taking into account only the data with the x-ordinate <= the current
     x-ordinate. It shows where there are large deviations contributing to the overall chi2
-    see eqn 8 - David, W.I. 2004. "Powder Diffraction: Least-Squares and Beyond." Journal of Research of the National Institute of Standards and Technology 109 (1): 107-23. https://doi.org/10.6028/jres.008.
+    see eqn 8 - David, W.I. 2004. "Powder Diffraction: Least-Squares and Beyond." Journal of Research of the National Institute of Standards and Technology 109 (1): 107-23.
+    https://doi.org/10.6028/jres.008.
+
+    chi^2 = (Rwp / Rexp)^2 = gof^2
+
     :param cifpat: dictionary representation of the cif pattern you want
     :param yobs_dataname: dataname of the yobs value
     :param ycalc_dataname: dataname of the ycalc value
@@ -255,6 +259,7 @@ def calc_cumchi2(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_dat
     """
     yobs = cifpat[yobs_dataname]
     ycalc = cifpat[ycalc_dataname]
+    N = len(cifpat[yobs_dataname])
 
     if ymod_dataname is not None:  # I don't know what ymod means yet...
         return [-1] * len(yobs)
@@ -263,10 +268,10 @@ def calc_cumchi2(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_dat
     else:
         yobs_dataname_err = yobs_dataname + "_err"
         yweight = 1 / (cifpat[yobs_dataname_err] ** 2)
-    return np.nancumsum(yweight * ((yobs - ycalc) ** 2))  # treats nan as 0
+    return np.nancumsum(yweight * ((yobs - ycalc) ** 2)) / N  # treats nan as 0
 
 
-def calc_rwp(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_dataname_err: str = None, ymod_dataname: str = None):
+def calc_rwp(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_dataname_err: str = None, ymod_dataname: str = None) -> float:
     """
     Calculate the Rwp statistic using the given yobs, ycalc, uncertinaty, and y_modifier.
     See Table 1.3 in "The Rietveld Method" by RA Young
@@ -275,13 +280,13 @@ def calc_rwp(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_datanam
     :param ycalc_dataname: dataname of the ycalc value
     :param yobs_dataname_err: defaults to the '_pd_proc_ls_weight'. If this isn't present, it goes for the "_err" column
     :param ymod_dataname: this currently does nothing.
-    :return: a numpy array with values of Rwp. It has the same length as yobs
+    :return: a float with value of Rwp.
     """
     yobs = cifpat[yobs_dataname]
     ycalc = cifpat[ycalc_dataname]
 
     if ymod_dataname is not None:  # I don't know what ymod means yet...
-        return [-1] * len(yobs)
+        return -1
     if yobs_dataname_err is None and "_pd_proc_ls_weight" in cifpat:
         yweight = cifpat["_pd_proc_ls_weight"]
     else:
@@ -293,12 +298,59 @@ def calc_rwp(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_datanam
     return np.sqrt(top / bottom)
 
 
-def theta2_from_min_max_inc(start:float, step:float, stop:float)->List[str]:
+def calc_rexp_approx(cifpat: dict, yobs_dataname: str, yobs_dataname_err: str = "", ymod_dataname: str = "") -> float:
+    """
+    Calculate the Rexp statistic using the given yobs, uncertinaty, and y_modifier.
+    See Table 1.3 in "The Rietveld Method" by RA Young
+
+    I am approximating N-P with N, as there should be many more datapoints than parameters.
+
+    :param cifpat: dictionary representation of the cif pattern you want
+    :param yobs_dataname: dataname of the yobs value
+    :param yobs_dataname_err: defaults to the '_pd_proc_ls_weight'. If this isn't present, it goes for the "_err" column
+    :param ymod_dataname: this currently does nothing.
+    :return: a float with value of Rexp.
+    """
+    yobs = cifpat[yobs_dataname]
+    N = len(cifpat[yobs_dataname])  # should strictly be N-P
+
+    if ymod_dataname:  # I don't know what ymod means yet...
+        return -1
+
+    if not yobs_dataname_err and "_pd_proc_ls_weight" in cifpat:
+        yweight = cifpat["_pd_proc_ls_weight"]
+    else:
+        yobs_dataname_err = yobs_dataname + "_err"
+        yweight = 1 / (cifpat[yobs_dataname_err] ** 2)
+    bottom = np.nansum(yweight * (yobs ** 2))
+    return np.sqrt(N / bottom)
+
+
+def calc_gof_approx(cifpat: dict, yobs_dataname: str, ycalc_dataname: str, yobs_dataname_err: str = None, ymod_dataname: str = None):
+    """
+    Calculate the goodness-of-fit statistic using the given yobs, ycalc, uncertinaty, and y_modifier.
+    See Table 1.3 in "The Rietveld Method" by RA Young
+
+    Should be using N-P, but there should be many more datapoints than parameters, so it should be pretty good.
+
+    :param cifpat: dictionary representation of the cif pattern you want
+    :param yobs_dataname: dataname of the yobs value
+    :param ycalc_dataname: dataname of the ycalc value
+    :param yobs_dataname_err: defaults to the '_pd_proc_ls_weight'. If this isn't present, it goes for the "_err" column
+    :param ymod_dataname: this currently does nothing.
+    :return: a float with values of gof.
+    """
+    rwp = calc_rwp(cifpat, yobs_dataname, ycalc_dataname, yobs_dataname_err, ymod_dataname)
+    rexp = calc_rexp_approx(cifpat, yobs_dataname, yobs_dataname_err, ymod_dataname)
+    return rwp / rexp
+
+
+def theta2_from_min_max_inc(start: float, step: float, stop: float) -> List[str]:
     num_points = int(round((stop - start) / step, 5)) + 1
     return [str(v) for v in np.linspace(start, stop, num_points)]
 
 
-def theta2_min_max_inc_expand(cifpat:CifBlock, val:str="meas")->None:
+def theta2_min_max_inc_expand(cifpat: CifBlock, val: str = "meas") -> None:
     MEAS_COL = ["_pd_meas_counts_total", "_pd_meas_intensity_total"]
     PROC_COL = ["_pd_proc_intensity_total", "_pd_proc_intensity_net"]
     MEAS_VALS = ["_pd_meas_2theta_range_min", "_pd_meas_2theta_scan", "_pd_meas_2theta_range_min", "_pd_meas_2theta_range_inc", "_pd_meas_2theta_range_max"]
@@ -327,7 +379,7 @@ def theta2_min_max_inc_expand(cifpat:CifBlock, val:str="meas")->None:
     cifpat.RemoveItem(maxval)
 
 
-def get_linked_structures_and_phase_ids(cifpat:CifFile, blockname_dict:dict)->Tuple[List[str], List[str]]:
+def get_linked_structures_and_phase_ids(cifpat: CifFile, blockname_dict: dict) -> Tuple[List[str], List[str]]:
     pd_phase_ids = []
     structures = []
     if "_pd_phase_block_id" in cifpat:  # then it is linked to other structures
@@ -345,19 +397,19 @@ def calc_dq_from_qd(dq):
     return 2. * np.pi / dq
 
 
-def calc_d_from_2theta(lam:float, th2):
+def calc_d_from_2theta(lam: float, th2):
     return lam / (2. * np.sin(th2 * np.pi / 360.))
 
 
-def calc_q_from_2theta(lam:float, th2):
+def calc_q_from_2theta(lam: float, th2):
     return 4. * np.pi * np.sin(th2 * np.pi / 360.) / lam
 
 
-def calc_2theta_from_d(lam:float, d):
+def calc_2theta_from_d(lam: float, d):
     return 2. * np.arcsin(lam / (2. * d)) * 180. / np.pi
 
 
-def add_hklds_to_cifpatstr(cifpatstr:dict, hkld_dict:dict)->None:
+def add_hklds_to_cifpatstr(cifpatstr: dict, hkld_dict: dict) -> None:
     """
     adds hkld information to a phase in the "str" section of a cif dictionary
     :param cifpatstr:  a dictionary of the form: cifpat["str"][phase_id]
@@ -368,7 +420,7 @@ def add_hklds_to_cifpatstr(cifpatstr:dict, hkld_dict:dict)->None:
         cifpatstr[refln] = hkld_dict[idx]
 
 
-def calc_dataname_and_err(cifpat:dict, dataname:str, default_error:str="zero") -> None:
+def calc_dataname_and_err(cifpat: dict, dataname: str, default_error: str = "zero") -> None:
     """
     gets a single value-as-string or list of values-as-strings and correctly turns them in to a float or lists of floats
     with an error term, if applicable
@@ -400,7 +452,7 @@ class ParseCIF:
                        "_pd_meas_2theta_range_inc", "_pd_proc_2theta_range_inc",  # these are here to capture blocks with start step stop values
                        "d", "q"]  # "_pd_proc_wavelength",
 
-    # These are datanames that fit correspond to the data you are modelling
+    # These are datanames that fit corresspond to the data you are modelling
     # these are also the only ones that could have an error
     OBSERVED_Y_LIST = ["_pd_meas_counts_total", "_pd_meas_intensity_total",
                        "_pd_proc_intensity_total", "_pd_proc_intensity_net"]
@@ -420,10 +472,11 @@ class ParseCIF:
 
     COMPLETE_Y_LIST = OBSERVED_Y_LIST + BACKGROUND_Y_LIST + CALCULATED_Y_LIST
 
-    NICE_TO_HAVE_DATANAMES = ['_diffrn_radiation_wavelength', "_cell_measurement_wavelength",
+    NICE_TO_HAVE_DATANAMES = ["_diffrn_radiation_wavelength", "_cell_measurement_wavelength",
                               "_diffrn_ambient_temperature", "_cell_measurement_temperature",
                               "_diffrn_ambient_pressure", "_cell_measurement_pressure",
-                              "_pd_meas_datetime_initiated"]
+                              "_refine_ls_goodness_of_fit_all", "_pd_proc_ls_prof_wr_factor",
+                              "_pd_proc_ls_prof_wr_expected", "_pd_meas_datetime_initiated"]
 
     DATANAMES_THAT_SHOULD_BE_NUMERIC = ["_pd_phase_mass_%", "_refln_d_spacing"] + \
                                        COMPLETE_X_LIST + COMPLETE_Y_LIST + MODIFIER_Y_LIST + \
@@ -431,9 +484,13 @@ class ParseCIF:
 
     def __init__(self, ciffilename, scantype="flex", grammar="1.1", scoping="dictionary", permissive=False):
         print(f"Now reading {ciffilename}. This may take a while.")
-        self.ciffile:CifFile = ReadCif(ciffilename, scantype=scantype, grammar=grammar, scoping=scoping, permissive=permissive)
-        self.ncif:dict = {}  # this will be the cif file with pattern information only
-        self.cif:dict = {}
+        self.ciffile: CifFile = ReadCif(ciffilename, scantype=scantype, grammar=grammar, scoping=scoping, permissive=permissive)
+        self.ncif: dict = {}  # this will be the cif file with pattern information only
+        self.cif: dict = {}
+
+
+        if self.ciffile is None:
+            raise ValueError("CIF file is empty.")
 
         self._remove_empty_items()
         self._make_up_block_id()
@@ -444,9 +501,10 @@ class ParseCIF:
         self._make_just_patterns_cif()
         self._make_cif_numeric()
         self._calc_extra_data()
+        self._put_phase_mass_pct_in_strs()
         self._rename_datablock_from_blockid()
 
-    def _remove_empty_items(self)->None:
+    def _remove_empty_items(self) -> None:
         """
         Checks every item in each block and if all the values are '?' or '.', it removes them from the cif
         :return: none. alters in place
@@ -470,7 +528,7 @@ class ParseCIF:
                 else:
                     print("How did we even get here?")
 
-    def _make_up_block_id(self)->None:
+    def _make_up_block_id(self) -> None:
         """
         Some people confuse the _pd_block_id and the data_ blockname.
         This goes and adds a _pd_block_id to every block that doesn't have an id such that _pd_block_id == blockname
@@ -480,7 +538,7 @@ class ParseCIF:
             cifblk = self.ciffile[block]
             cifblk["_pd_block_id"] = block if "_pd_block_id" not in cifblk else cifblk["_pd_block_id"]
 
-    def _expand_2theta_min_max_inc(self)->None:
+    def _expand_2theta_min_max_inc(self) -> None:
         """
         Expand _pd_meas_2theta_range_min/max/inc into explicit values of _pd_meas_2theta_scan, and
         _pd_proc_2theta_range_min/max/inc into _pd_proc_2theta_corrected. The min/max/inc dataitems are
@@ -493,7 +551,7 @@ class ParseCIF:
             theta2_min_max_inc_expand(cifpat, "meas")
             theta2_min_max_inc_expand(cifpat, "proc")
 
-    def _expand_multiple_dataloops(self)->None:
+    def _expand_multiple_dataloops(self) -> None:
         """
         This looks for patterns which have multiple loops containing diffraction data
         This clones the block as many times as there are loops, with each clone containing
@@ -541,7 +599,7 @@ class ParseCIF:
                 self.ciffile.NewBlock(new_pattern_name, insert_me)
                 self.ciffile[new_pattern_name]["_pd_block_id"] = new_block_prefix + get_from_cif(self.ciffile[new_pattern_name], "_pd_block_id", new_pattern_name)
 
-    def _get_hkl_ticks(self)->None:
+    def _get_hkl_ticks(self) -> None:
         """
         Go through a pattern and look for linked structures. Get hkld information from either the linked structures
         or from the in reflection listing in the pattern block if there is a _pd_refln_phase_id data name
@@ -573,7 +631,7 @@ class ParseCIF:
                     for phase_id, structure in zip(pd_phase_ids, structures):
                         cifstr = self.cif[structure]
                         hkld_dict = get_hkld_ids(cifstr)
-                        if hkld_dict is None:
+                        if not hkld_dict:
                             continue
                         add_hklds_to_cifpatstr(cifpat["str"][phase_id], hkld_dict)
                 elif "_pd_refln_phase_id" in cifpat:
@@ -587,13 +645,13 @@ class ParseCIF:
                             print(f"No hkls found for phase {phase_id}.")
             elif "_refln_d_spacing" in cifpat:  # there is no phase_block_id and so only a single phase's worth of tick marks
                 hkld_dict = get_hkld_ids(cifpat)
-                if hkld_dict is None:
+                if not hkld_dict:
                     continue
                 cifpat["str"] = {"1": {}}
                 add_hklds_to_cifpatstr(cifpat["str"]["1"], hkld_dict)
                 cifpat["str"]["1"]["_pd_phase_name"] = get_from_cif(cifpat, "_pd_phase_name", default="1")
 
-    def _get_nice_to_have_information(self)->None:
+    def _get_nice_to_have_information(self) -> None:
         """
         This traverses the linked structures, and then the "other" blocks, to look for information that is nice to have with
         the pattern
@@ -612,7 +670,7 @@ class ParseCIF:
                 if dataname not in cifpat or cifpat[dataname] in [".", "?"]:  # then need to look for it in different places
                     get_dataname_into_pattern(self.cif, pattern, dataname, structures, others)
 
-    def _make_just_patterns_cif(self)->None:
+    def _make_just_patterns_cif(self) -> None:
         """
         Goes through all of the blocks in the cif dictionary and only copies out the onces which contain diffraction patterns.
         All the other necessary information should already be in the "str" entry.
@@ -631,10 +689,12 @@ class ParseCIF:
                     continue
                 if dataname in self.OBSERVED_Y_LIST:
                     calc_dataname_and_err(cifpat, dataname, default_error="sqrt")
+                elif dataname in ("_pd_phase_mass_%"):
+                    calc_dataname_and_err(cifpat, dataname, default_error="zero")  # for single phase, so there is no error on the wt%
                 else:
                     calc_dataname_and_err(cifpat, dataname, default_error="none")
 
-    def _calc_extra_data(self)->None:
+    def _calc_extra_data(self) -> None:
         for pattern in self.ncif:
             cifpat = self.ncif[pattern]
             if "_pd_proc_d_spacing" not in cifpat and "_pd_proc_recip_len_Q" in cifpat:
@@ -668,11 +728,23 @@ class ParseCIF:
                         cifstr["refln_hovertext"] = [f"{h} {k} {l}" for h, k, l in
                                                      zip(cifstr['_refln_index_h'], cifstr['_refln_index_k'], cifstr["_refln_index_l"])]
 
-    def _rename_datablock_from_blockid(self)->None:
+    def _put_phase_mass_pct_in_strs(self) -> None:
+        for pattern in self.ncif:
+            cifpat = self.ncif[pattern]
+            if "_pd_phase_mass_%" not in cifpat or "str" not in cifpat:
+                continue
+            phase_ids = cifpat["_pd_phase_id"]
+            qpas = cifpat["_pd_phase_mass_%"]
+            qpa_errs = cifpat["_pd_phase_mass_%_err"]
+            for phase, qpa, err in zip(phase_ids, qpas, qpa_errs):
+                cifpat["str"][phase]["_pd_phase_mass_%"] = qpa
+                cifpat["str"][phase]["_pd_phase_mass_%_err"] = err
+
+    def _rename_datablock_from_blockid(self) -> None:
         """
         A lot of the pdCIF revolves around block_id, and not the dataname.
         This renames the dataname as the block_id to enable easier lookups.
-        This needs to be done as a ductionary, and not a PyCifRW, as PyCIFRW
+        This needs to be done as a dictionary, and not a PyCifRW, as PyCIFRW
         forces datablock names to be lowercase.
         :return:
         """
@@ -690,46 +762,8 @@ class ParseCIF:
 
 
 if __name__ == "__main__":
-    # filename = r"..\..\data\forJames_before.cif"
-    # filename = r"..\..\data\ideal_condensed.cif"
-    # filename = r"..\..\data\ideal_strsWithHKLs_condensed.cif"
-    # filename = r"..\..\data\ALUMINA.cif"
     filename = r"c:\data\La2Ti2O7-n-883C-mono.cif"
-    # filename = r"c:\data\yh5011sup1.cif"
-    # filename = r"c:\data\Nd2Ti2O7-x-1456C-mono_fixed.cif"
-    # filename = r"..\..\data\NISI.cif"
-    # filename = r"..\..\data\ideal_5patterns.cif"
-    # filename = r"..\..\data\pam\ws5072ibuprofen_all.cif"
-    # filename = r"..\..\data\pam\mag_cif_testfile_modified.cif"
-    # filename = r"..\..\data\simon\cifs\cu31501sup2.rtv.combined.cif"
-    #
-    # _diffrn_wavelength = two values for ka1 ka2
-
-    import os
-
-    # #18
-    # filename = files[189]
-
-    # os.system("start " + filename)
     cf = ParseCIF(filename)
     cifd = cf.get_processed_cif()
     print("This is the end of the file:")
     pretty(cifd, print_values=True)
-    # print(filename)
-    # print(files[18])
-    # 18 could not convert string to float: 'YES' C:/Users/184277j/Documents/GitHub/pdCIFplotter/data/simon/cifs/hr0041isup4.rtv.combined.cif
-
-#
-# for i in range(len(files)):
-#     filename = files[i]
-#     try:
-#         cf = ParseCIF(filename)
-#         # print(i)
-#     except Exception as e:
-#         print(f"{i}\t{e}\t{filename}")
-#         continue
-
-
-# print(cf.rwp("pattern_0", "_pd_meas_intensity_total", "_pd_calc_intensity_total", "_pd_proc_ls_weight"))
-
-# print(type(cf))
