@@ -1,28 +1,36 @@
-from numpy import ndarray
-
 from pdCIFplotter import parse_cif
 import numpy as np
 import math
-import matplotlib.figure as mf
-import matplotlib.axes as ma
-import matplotlib.pyplot as plt
+from matplotlib.figure import Figure
+from matplotlib.axes import Axes
+
 import matplotlib.colors as mc  # a lot of colour choices in here to use
 # from timeit import default_timer as timer  # use as start = timer() ...  end = timer()
 import mplcursors
 from typing import List, Tuple, Any
 
-DEBUG = True
+DEBUG = False
 
 
 def debug(*args):
     if DEBUG:
         print(*args)
 
+#colour thing
+_by_hsv = sorted((tuple(mc.rgb_to_hsv(mc.to_rgb(color))), name) for name, color in mc.CSS4_COLORS.items())
+
+
+# Font related things
+FONT_FAMILY = ["sans-serif", "serif", "cursive", "fantasy", "monospace"]
+FONT_SIZES = [6, 7, 8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 26, 28, 36, 48, 72]
+FONT_COLOURS = [name for hsv, name in _by_hsv]
+FONT_WEIGHTS = ['normal', 'bold', 'light']
+FONT_STYLES = ['normal', 'italic', 'oblique']
+
 
 # LINE_MARKER_COLORS = list(mc.CSS4_COLORS.keys())
 # from here: https://matplotlib.org/stable/gallery/color/named_colors.html
-_by_hsv = sorted((tuple(mc.rgb_to_hsv(mc.to_rgb(color))), name) for name, color in mc.CSS4_COLORS.items())
-LINE_MARKER_COLORS = [name for hsv, name in _by_hsv]
+LINE_MARKER_COLORS = FONT_COLOURS #[name for hsv, name in _by_hsv]
 MARKER_STYLES: List[str] = [None, ".", "o", "s", "*", "+", "x", "D"]
 LINE_STYLES: List[str] = ["solid", "None", "dashed", "dashdot", "dotted"]
 LINE_MARKER_SIZE: List[str] = [str(s) for s in range(1, 10)]
@@ -39,6 +47,12 @@ for c in SURFACE_COLOR_MAPS_SOURCE:
 TABLEAU_COLORS = mc.TABLEAU_COLORS
 TABLEAU_COLOR_VALUES: List[str] = list(TABLEAU_COLORS.values())
 
+def get_dict_value(dic, key, default=None):
+    try:
+        return dic[key]
+    except KeyError:
+        return default
+
 
 def _x_axis_title(x_ordinate: str, wavelength: float = None) -> str:
     if wavelength is None:
@@ -114,6 +128,108 @@ def _scale_xyz_title(xtitle: str, ytitle: str, ztitle: str, axis_scale: dict) ->
     return _scale_x_title(xtitle, axis_scale), _scale_y_title(ytitle, axis_scale), _scale_z_title(ztitle, axis_scale)
 
 
+def add_hovertext_to_each_point(artists, hovertexts):
+    # https://stackoverflow.com/a/58350037/36061
+    hover_dict = dict(zip(artists, hovertexts))
+    mplcursors.cursor(artists, hover=mplcursors.HoverMode.Transient).connect(
+        "add", lambda sel: sel.annotation.set_text(hover_dict[sel.artist][sel.index]))
+
+
+# mplcursors.cursor(stack_artists, hover=mplcursors.HoverMode.Transient).connect("add", lambda sel: sel.annotation.set_text(sel.artist.get_label()))
+
+# https://stackoverflow.com/a/30734735/36061
+def array_strictly_increasing_or_equal(a):
+    return np.all(a[1:] >= a[:-1])
+
+
+def array_strictly_decreasing_or_equal(a):
+    return np.all(a[1:] <= a[:-1])
+
+
+def rescale_val(val: float, old_scale: dict, new_scale: dict, ordinate: str):
+    if old_scale[ordinate] == "log":
+        val = math.copysign(1, val) * 10 ** abs(val)
+    elif old_scale[ordinate] == "sqrt":
+        val = math.copysign(1, val) * val ** 2
+    return _scale_ordinate(new_scale, val, ordinate)
+
+
+def get_first_different_kv_pair(old_dict: dict, new_dict: dict):
+    for k, v in old_dict.items():
+        if new_dict[k] != v:
+            return k, v
+
+
+def get_zoomed_data_min_max(ax, x_lim: Tuple, y_lim: Tuple):
+    xmin = x_lim[0]
+    xmax = x_lim[1]
+    ymin = y_lim[0]
+    ymax = y_lim[1]
+    xs = []
+    ys = []
+
+    if xmin > xmax:
+        xmin, xmax = xmax, xmin
+    for line in ax.lines:
+        xa = line.get_xdata()
+        ya = line.get_ydata()
+        for x, y in zip(xa, ya):
+            if xmin < x < xmax:
+                xs.append(x)
+                ys.append(y)
+    xmin = min(xs)
+    xmax = max(xs)
+    ymin = max(ymin, min(ys))
+    ymax = min(ymax, max(ys))
+
+    return (xmin, xmax), (ymin / 1.04, ymax * 1.04)
+
+
+def isclose_listlike(a, b, rel_tol=1e-09, abs_tol=0.0):
+    if a is None or b is None:
+        return False
+    if len(a) != len(b):
+        return False
+    return all(
+        math.isclose(av, bv, rel_tol=rel_tol, abs_tol=abs_tol)
+        for av, bv in zip(a, b)
+    )
+
+
+def make_subtitle_string(cifpat, s="", yobs: str = "", ycalc: str = "") -> str:
+    def add_string(add_me, s, brackets=False):
+        if s and brackets:
+            return f"{s} ({add_me})"
+        elif s:
+            return f"{s}; {add_me}"
+        else:
+            return add_me
+
+    print_Rfactor = ((yobs and ycalc) and (yobs != "None" and ycalc != "None")) or ("_pd_proc_ls_prof_wr_factor" in cifpat or "_refine_ls_goodness_of_fit_all" in cifpat)
+
+    if "_pd_meas_datetime_initiated" in cifpat:
+        s = add_string(cifpat['_pd_meas_datetime_initiated'], s)
+    if "_diffrn_ambient_temperature" in cifpat:
+        s = add_string(f"{cifpat['_diffrn_ambient_temperature']} K", s)
+    if "_diffrn_ambient_pressure" in cifpat:
+        s = add_string(f"{cifpat['_diffrn_ambient_pressure']} kPa", s)
+    if print_Rfactor:
+        s = add_string("GoF = ", s)
+    if "_refine_ls_goodness_of_fit_all" in cifpat:
+        s += f"{cifpat['_refine_ls_goodness_of_fit_all']:.3f}"
+    if (yobs and ycalc) and (yobs != "None" and ycalc != "None"):
+        gof = parse_cif.calc_gof_approx(cifpat, yobs, ycalc)
+        s = add_string(f"{gof:.3f}", s, brackets=True)
+    if print_Rfactor:
+        s = add_string("Rwp = ", s)
+    if "_pd_proc_ls_prof_wr_factor" in cifpat:
+        s += f"{cifpat['_pd_proc_ls_prof_wr_factor'] * 100:.2f}%"
+    if (yobs and ycalc) and (yobs != "None" and ycalc != "None"):
+        rwp = parse_cif.calc_rwp(cifpat, yobs, ycalc)
+        s = add_string(f"{rwp * 100:.2f}%", s, brackets=True)
+    return s
+
+
 class PlotCIF:
     hkl_x_ordinate_mapping = {"_pd_proc_d_spacing": "_refln_d_spacing", "d": "_refln_d_spacing", "q": "refln_q", "_pd_meas_2theta_scan": "refln_2theta",
                               "_pd_proc_2theta_corrected": "refln_2theta"}
@@ -134,7 +250,17 @@ class PlotCIF:
                                   "x_data": None, "y_data": None, "z_data": None, "z_norm": None,
                                   "plot_list": []}
 
+        self.previous_single_plot_state = {"pattern": "", "change_data": False,
+                                           "x_ordinate": "", "y_ordinates": ["None"],
+                                           "plot_hkls": {}, "plot_diff": False, "plot_cchi2": False, "plot_norm_int": False,
+                                           "axis_scale": {},
+                                           "data_x_lim": None, "data_y_lim": None,
+                                           "zoomed_x_lim": None, "zoomed_y_lim": None}
+
         self.cif: dict = cif
+        self.dpi = 100
+
+        self.qpa = self.get_all_qpa()
 
     def get_all_xy_data(self, x_ordinate: str, y_ordinate: str) -> Tuple[List, List, List[str]]:
         """
@@ -158,7 +284,7 @@ class PlotCIF:
             plot_list.append(pattern)
         return xs, ys, plot_list
 
-    def get_all_scaled_xy_data(self, x_ordinate: str, y_ordinate: str, axis_scale:dict) -> Tuple[List, List, List[str]]:
+    def get_all_scaled_xy_data(self, x_ordinate: str, y_ordinate: str, axis_scale: dict) -> Tuple[List, List, List[str]]:
         """
         Give an x and y-ordinate, return arrays holding all x ad y data from all patterns in the cif
         that have both of those ordinates. Also return a list of the pattern names which where included
@@ -180,7 +306,6 @@ class PlotCIF:
             plot_list.append(pattern)
         return xs, ys, plot_list
 
-
     def get_xy_ynorm_data(self, x_ordinate: str, y_ordinate: str) -> List:
         """
         Give an x and y-ordinate, return arrays holding y_norm values from all patterns in the cif
@@ -203,7 +328,7 @@ class PlotCIF:
             y_norms.append(y_norm)
         return y_norms
 
-    def get_all_xyz_znorm_data(self, x_ordinate: str, y_ordinate: str, z_ordinate: str, z_norm_ordinate: str) -> tuple[Any, Any, ndarray, ndarray, list[Any]]:
+    def get_all_xyz_znorm_data(self, x_ordinate: str, y_ordinate: str, z_ordinate: str, z_norm_ordinate: str) -> Tuple[Any, Any, np.ndarray, np.ndarray, List[Any]]:
         # need to construct a single array for each x, y, z, by looping through only those
         # patterns which have the ordinates necessary to make the piccie I want to see.
         xs = []
@@ -266,22 +391,50 @@ class PlotCIF:
         zn = np.array(znorms)
         return xx, yy, zz, zn, plot_list
 
-    def single_update_plot(self, pattern: str,
-                           x_ordinate: str, y_ordinates: list,
+    def get_all_qpa(self) -> dict:
+        all_phases = set()
+        # get all phase names
+        for pattern in self.cif:
+            cifpat = self.cif[pattern]
+            if "str" not in cifpat:
+                continue
+            for phase in cifpat["str"]:
+                all_phases.add(cifpat["str"][phase]["_pd_phase_name"])
+        d = {phase: [] for phase in all_phases}
+        # get all phase wt%
+        for i, pattern in enumerate(self.cif, start=1):
+            cifpat = self.cif[pattern]
+            if "str" not in cifpat:
+                continue
+            for phase in cifpat["str"]:
+                try:
+                    qpa = cifpat["str"][phase]["_pd_phase_mass_%"]
+                except KeyError:
+                    qpa = None
+                phase_name = cifpat["str"][phase]["_pd_phase_name"]
+                d[phase_name].append(qpa)
+            for phase, qpa_list in d.items():
+                if len(qpa_list) != i:
+                    d[phase].append(np.nan)
+        return d
+
+    def single_update_plot(self, pattern: str, x_ordinate: str, y_ordinates: List[str],
                            plot_hkls: dict, plot_diff: bool, plot_cchi2: bool, plot_norm_int: bool,
-                           axis_scale: dict,
-                           fig: mf.Figure) -> mf.Figure:
+                           axis_scale: dict, fontdict: dict,
+                           fig: Figure, ax: Axes) -> Tuple[Figure, Axes, Tuple, Tuple]:
         # todo: look at https://stackoverflow.com/a/63152341/36061 for idea on zooming
-        dpi = plt.gcf().get_dpi()
-        single_height_px = fig.get_size_inches()[1] * dpi if fig is not None else 382  # this is needed for the hkl position calculations
+        single_height_px = fig.get_size_inches()[1] * self.dpi if fig is not None else 382  # this is needed for the hkl position calculations
         # if single_fig is None or single_ax is None:
-        if fig is not None:
-            plt.close(fig)
-        fig, ax = plt.subplots(1, 1)
-        fig = plt.gcf()
-        fig.set_size_inches(self.canvas_x / float(dpi), self.canvas_y / float(dpi))
-        fig.set_tight_layout(True)
-        plt.margins(x=0)
+        zoomed_x_lim = None
+        zoomed_y_lim = None
+        if fig:
+            zoomed_x_lim = ax.get_xlim()
+            zoomed_y_lim = ax.get_ylim()
+            fig.clear()
+        fig = Figure(figsize=(6, 3), dpi=self.dpi)
+        ax = fig.add_subplot()
+        # fig.set_tight_layout(True)  # https://github.com/matplotlib/matplotlib/issues/21970 https://github.com/matplotlib/matplotlib/issues/11059
+        ax.margins(x=0)
 
         cifpat = self.cif[pattern]
         x = _scale_x_ordinate(cifpat[x_ordinate], axis_scale)
@@ -318,7 +471,7 @@ class PlotCIF:
                 if y_name == "Diff":
                     offset = min_plot - np.nanmax(y)
                     y += offset
-                    # this is to plot the 'sero' line for the diff plot
+                    # this is to plot the 'zero' line for the diff plot
                     ax.plot(x, [offset] * len(x), color="black", marker=None, linestyle=(0, (5, 10)), linewidth=1)  # "loosely dashed"
 
                 label = f" {y_name}" if not plot_norm_int else f" {y_name} (norm.)"
@@ -327,6 +480,7 @@ class PlotCIF:
                         linestyle=self.single_y_style[y_type]["linestyle"], linewidth=self.single_y_style[y_type]["linewidth"],
                         markersize=float(self.single_y_style[y_type]["linewidth"]) * 3
                         )
+
                 # keep track of min and max to plot hkl ticks and diff correctly
                 min_plot = min(min_plot, np.nanmin(y))
                 max_plot = max(max_plot, np.nanmax(y))
@@ -334,14 +488,18 @@ class PlotCIF:
                     cchi2_zero = min_plot
 
         if plot_hkls["above"] or plot_hkls["below"]:
-            single_hovertexts, single_hkl_artists = self.plot_hkls(plot_hkls["below"], cifpat, x_ordinate, x, ys, axis_scale, min_plot, max_plot, 0, True, dpi, single_height_px, ax)
-            self.add_hovertext_to_hkls(single_hkl_artists, single_hovertexts)
+            single_hovertexts, single_hkl_artists = self.plot_hkls(plot_hkls["below"], cifpat,
+                                                                   x_ordinate, x, ys, axis_scale,
+                                                                   min_plot, max_plot, 0, True, self.dpi,
+                                                                   single_height_px, ax)
+            add_hovertext_to_each_point(single_hkl_artists, single_hovertexts)
 
         if plot_cchi2:
-            ax2 = self.single_plot_cchi2(cifpat, x, [y_ordinates[0], y_ordinates[1]], axis_scale, cchi2_zero, ax)
+            flip_cchi2 = x_ordinate in {"d", "_pd_proc_d_spacing"}
+            ax2 = self.single_plot_cchi2(cifpat, x, [y_ordinates[0], y_ordinates[1]], axis_scale, cchi2_zero, flip_cchi2, fontdict, ax)
 
         if not plot_cchi2:
-            plt.legend(frameon=False, loc='upper right')  # loc='best')
+            ax.legend(frameon=False, loc='upper right', prop=fontdict["legend"], labelcolor=fontdict["legendcolor"]["color"])  # loc='best')
 
         if plot_norm_int:
             y_axis_title = "Normalised counts"
@@ -353,24 +511,109 @@ class PlotCIF:
         wavelength = parse_cif.get_from_cif(cifpat, "wavelength")
         x_axis_title, y_axis_title = _scale_xy_title(_x_axis_title(x_ordinate, wavelength), y_axis_title, axis_scale)
 
+        ax.set_xlabel(x_axis_title, fontdict=fontdict["xlabel"])
+        ax.set_ylabel(y_axis_title, fontdict=fontdict["ylabel"])
+        fig.suptitle(pattern, x=fig.subplotpars.left, horizontalalignment="left", fontproperties={"size": fontdict["title"]["size"]}, fontdict=fontdict["title"])
+        fig.subplots_adjust(top=0.9)
+
+        subtitle = make_subtitle_string(cifpat, yobs=y_ordinates[0], ycalc=y_ordinates[1])
+        if subtitle:
+            ax.set_title(subtitle, loc="left", fontdict=fontdict["subtitle"])
+
+        for label in ax.get_xticklabels():
+            label.update(fontdict["xticklabel"])
+        for label in ax.get_yticklabels():
+            label.update(fontdict["yticklabel"])
+
         if x_ordinate in {"d", "_pd_proc_d_spacing"}:
-            plt.gca().invert_xaxis()
-
-        ax.set_xlabel(x_axis_title)
-        ax.set_ylabel(y_axis_title)
-        plt.title(pattern, loc="left")
-
+            ax.invert_xaxis()
         # https://stackoverflow.com/a/30506077/36061
         if plot_cchi2:
             ax.set_zorder(ax2.get_zorder() + 1)
             ax.patch.set_visible(False)
 
-        return fig
+        # now I need to set up all the checks to see if I want to push new views onto the home stack
+        #  and how to reset the zoom.
+        reset_zoomed_to_plt_x_min = False
+        reset_zoomed_to_plt_x_max = False
+        reset_zoomed_to_plt_y_min = False
+        reset_zoomed_to_plt_y_max = False
+        data_x_lim = ax.get_xlim()
+        data_y_lim = ax.get_ylim()
+        zoomed_x_lim = ax.get_xlim() if not zoomed_x_lim else zoomed_x_lim
+        zoomed_y_lim = ax.get_ylim() if not zoomed_y_lim else zoomed_y_lim
+
+        # here go the rules on changing zoom to match the current data
+        if (
+            self.previous_single_plot_state["pattern"] != pattern
+            # if the zoom view of the data is the entire data, then the view of the new data is the
+            #  entire view of the new data, irrespective of the limits of the previous zoom
+            and isclose_listlike(self.previous_single_plot_state["data_x_lim"], zoomed_x_lim)
+            and isclose_listlike(self.previous_single_plot_state["data_y_lim"], zoomed_y_lim)
+        ):
+            reset_zoomed_to_plt_x_min = True
+            reset_zoomed_to_plt_x_max = True
+            reset_zoomed_to_plt_y_min = True
+            reset_zoomed_to_plt_y_max = True
+        if self.previous_single_plot_state["x_ordinate"] != x_ordinate:
+            reset_zoomed_to_plt_x_min = True
+            reset_zoomed_to_plt_x_max = True
+        if self.previous_single_plot_state["y_ordinates"] != y_ordinates:
+            reset_zoomed_to_plt_y_min = True
+            reset_zoomed_to_plt_y_max = True
+        if self.previous_single_plot_state["plot_diff"] != plot_diff:
+            reset_zoomed_to_plt_y_min = True
+        if self.previous_single_plot_state["plot_hkls"] != plot_hkls:
+            reset_zoomed_to_plt_y_min = True
+
+        if self.previous_single_plot_state["plot_norm_int"] != plot_norm_int:
+            # _, (ymin, ymax) = get_zoomed_data_min_max(ax, zoomed_x_lim, data_y_lim)
+            # yrange = (ymax - ymin)
+            # ymid = yrange / 2
+            # yrange = (yrange * 1.07) / 2
+            zoomed_y_lim = data_y_lim
+        if self.previous_single_plot_state["axis_scale"] not in [axis_scale, {}]:
+            ordinate, _ = get_first_different_kv_pair(self.previous_single_plot_state["axis_scale"], axis_scale)
+            if ordinate == "x":
+                zoomed_xmin = rescale_val(zoomed_x_lim[0], self.previous_single_plot_state["axis_scale"], axis_scale, ordinate)
+                zoomed_xmax = rescale_val(zoomed_x_lim[1], self.previous_single_plot_state["axis_scale"], axis_scale, ordinate)
+                zoomed_x_lim = (zoomed_xmin, zoomed_xmax)
+            elif ordinate == "y":
+                zoomed_ymin = rescale_val(zoomed_y_lim[0], self.previous_single_plot_state["axis_scale"], axis_scale, ordinate)
+                zoomed_ymax = rescale_val(zoomed_y_lim[1], self.previous_single_plot_state["axis_scale"], axis_scale, ordinate)
+                zoomed_y_lim = (zoomed_ymin, zoomed_ymax)
+
+        if reset_zoomed_to_plt_x_min:
+            zoomed_x_lim = (data_x_lim[0], zoomed_x_lim[1])
+        if reset_zoomed_to_plt_x_max:
+            zoomed_x_lim = (zoomed_x_lim[0], data_x_lim[1])
+        if reset_zoomed_to_plt_y_min:
+            zoomed_y_lim = (data_y_lim[0], zoomed_y_lim[1])
+        if reset_zoomed_to_plt_y_max:
+            zoomed_y_lim = (zoomed_y_lim[0], data_y_lim[1])
+
+        # Now it's time to set up the previous-state dictionary for the next time this function is called.
+        #  Need to do it right at the end so nothing else changes
+        self.previous_single_plot_state["pattern"] = pattern
+        self.previous_single_plot_state["x_ordinate"] = x_ordinate
+        self.previous_single_plot_state["y_ordinates"] = y_ordinates
+        self.previous_single_plot_state["plot_hkls"] = plot_hkls
+        self.previous_single_plot_state["plot_diff"] = plot_diff
+        self.previous_single_plot_state["plot_cchi2"] = plot_cchi2
+        self.previous_single_plot_state["plot_norm_int"] = plot_norm_int
+        self.previous_single_plot_state["axis_scale"] = axis_scale
+        self.previous_single_plot_state["data_x_lim"] = ax.get_xlim()
+        self.previous_single_plot_state["data_y_lim"] = ax.get_ylim()
+        self.previous_single_plot_state["zoomed_x_lim"] = zoomed_x_lim
+        self.previous_single_plot_state["zoomed_y_lim"] = zoomed_y_lim
+
+        return fig, ax, zoomed_x_lim, zoomed_y_lim
+
 
     def plot_hkls(self, plot_below: bool, cifpat: dict, x_ordinate: str, x, ys,
                   axis_scale: dict, y_min: float, y_max: float, hkl_y_offset: float,
                   single_plot: bool,
-                  dpi: int, single_height_px: int, ax: ma.Axes):
+                  dpi: int, single_height_px: int, ax: Axes):
 
         def interp(_hkl_x, _x, _y):
             return np.interp(_hkl_x, _x, _y, left=float("nan"), right=float("nan"))
@@ -384,9 +627,8 @@ class PlotCIF:
         hkl_tick_vertical_spacing = (((y_range / (single_height_px - hkl_markersize_px * num_hkl_rows)) * single_height_px) - y_range) / num_hkl_rows
 
         hkl_x_ordinate = PlotCIF.hkl_x_ordinate_mapping[x_ordinate]
-        for i, phase in enumerate(cifpat["str"].keys()):
+        for i, phase in enumerate(cifpat["str"]):
             if hkl_x_ordinate not in cifpat["str"][phase]:
-                print(f"Couldn't find {hkl_x_ordinate} in {phase=}.")
                 continue
             hkl_x = _scale_x_ordinate(cifpat["str"][phase][hkl_x_ordinate], axis_scale)
             if plot_below:
@@ -408,10 +650,12 @@ class PlotCIF:
                 else:
                     hkl_y = np.maximum(interp(hkl_x, x, ycalc), interp(hkl_x, x, yobs))
 
+            phase_wt_pct = f'– {cifpat["str"][phase]["_pd_phase_mass_%"]} wt%' if "_pd_phase_mass_%" in cifpat["str"][phase] else ""
+
             hkl_y = hkl_y * scalar + hkl_y_offset
             idx = i % len(TABLEAU_COLOR_VALUES)
             phasename = cifpat["str"][phase]["_pd_phase_name"] if "_pd_phase_name" in cifpat["str"][phase] else phase
-            hkl_tick, = ax.plot(hkl_x, hkl_y, label=" " + phasename, marker=markerstyle, linestyle="none", markersize=hkl_markersize_pt,
+            hkl_tick, = ax.plot(hkl_x, hkl_y, label=f" {phasename} {phase_wt_pct}", marker=markerstyle, linestyle="none", markersize=hkl_markersize_pt,
                                 color=TABLEAU_COLOR_VALUES[idx])
             hkl_artists.append(hkl_tick)
             if "refln_hovertext" in cifpat["str"][phase]:
@@ -423,27 +667,8 @@ class PlotCIF:
 
         return hovertexts, hkl_artists
 
-    def add_hovertext_to_hkls(self, hkl_artists, hovertexts):
-        # https://stackoverflow.com/a/58350037/36061
-        hkl_hover_dict = dict(zip(hkl_artists, hovertexts))
-        mplcursors.cursor(hkl_artists, hover=mplcursors.HoverMode.Transient).connect(
-            "add", lambda sel: sel.annotation.set_text(hkl_hover_dict[sel.artist][sel.index]))
-
-    def plot_norm_int_to_err(self, cifpat: dict, x, norm_int_to_err_y_ordinate: str, axis_scale: dict, ax: ma.Axes) -> None:
-        y = cifpat[norm_int_to_err_y_ordinate]
-        if "_pd_proc_ls_weight" in cifpat:
-            y_norm = y ** 2 * cifpat["_pd_proc_ls_weight"]
-        else:
-            y_norm = (y / cifpat[norm_int_to_err_y_ordinate + "_err"]) ** 2
-        y_norm = _scale_y_ordinate(y_norm, axis_scale)
-
-        ax.plot(x, y_norm, label=" Norm. int. to errors",
-                color=self.single_y_style["norm_int"]["color"], marker=self.single_y_style["norm_int"]["marker"],
-                linestyle=self.single_y_style["norm_int"]["linestyle"], linewidth=self.single_y_style["norm_int"]["linewidth"],
-                markersize=float(self.single_y_style["norm_int"]["linewidth"]) * 3
-                )
-
-    def single_plot_cchi2(self, cifpat: dict, x, cchi2_y_ordinates: List[str], axis_scale: dict, cchi2_zero: float, ax1: ma.Axes) -> ma.Axes:
+    def single_plot_cchi2(self, cifpat: dict, x, cchi2_y_ordinates: List[str], axis_scale: dict, cchi2_zero: float, flip_cchi2: bool,
+                          fontdict: dict, ax1: Axes) -> Axes:
         # https://stackoverflow.com/a/10482477/36061
         def align_cchi2(ax_1, v1, ax_2):
             """adjust cchi2 ylimits so that 0 in cchi2 axis is aligned to v1 in main axis"""
@@ -455,13 +680,32 @@ class PlotCIF:
             ax_2.set_ylim(miny2, maxy2)
 
         cchi2 = _scale_y_ordinate(parse_cif.calc_cumchi2(cifpat, cchi2_y_ordinates[0], cchi2_y_ordinates[1]), axis_scale)
-        rwp = parse_cif.calc_rwp(cifpat, cchi2_y_ordinates[0], cchi2_y_ordinates[1])
+
+        if array_strictly_increasing_or_equal(x) != array_strictly_increasing_or_equal(cchi2):
+            flip_cchi2 = True
+        else:
+            pass
+
+        if flip_cchi2:
+            # this keeps the differences between datapoints, but inverts their sign, so
+            #  the list ends up changing from increasing to decreasing, or vice versa.
+            #  This is done to counteract an inverted plotting axis so I can keep the
+            #  cchi2 plot increasing from left to right
+            ylag = np.diff(cchi2, prepend=cchi2[0])
+            ynew = np.zeros(len(cchi2))
+            ynew[0] = cchi2[-1]
+            for i in range(1, len(ynew)):
+                ynew[i] = ynew[i - 1] - ylag[i]
+            cchi2 = ynew
+
         ax2 = ax1.twinx()
-        ax2.plot(x, cchi2, label=f" c\u03C7\u00b2 - (Rwp = {rwp * 100:.2f}%)",
+
+        ax2.plot(x, cchi2, label=f" c\u03C7\u00b2",
                  color=self.single_y_style["cchi2"]["color"], marker=self.single_y_style["cchi2"]["marker"],
                  linestyle=self.single_y_style["cchi2"]["linestyle"], linewidth=self.single_y_style["cchi2"]["linewidth"],
                  markersize=float(self.single_y_style["cchi2"]["linewidth"]) * 3
                  )
+
         ax2.set_yticklabels([])
         ax2.set_yticks([])
         ax2.margins(x=0)
@@ -473,48 +717,48 @@ class PlotCIF:
         # ask matplotlib for the plotted objects and their labels
         lines, labels = ax1.get_legend_handles_labels()
         lines2, labels2 = ax2.get_legend_handles_labels()
-        ax2.legend(lines + lines2, labels + labels2, loc='upper right', frameon=False)
+        ax2.legend(lines + lines2, labels + labels2, loc='upper right', frameon=False, prop=fontdict["legend"], labelcolor=fontdict["legendcolor"]["color"])
 
         return ax2
 
     def stack_update_plot(self,
                           x_ordinate: str, y_ordinate: str, offset: float,
-                          plot_hkls: dict, plot_norm_int:dict,
-                          axis_scale: dict,
-                          fig: mf.Figure) -> mf.Figure:
-        dpi = plt.gcf().get_dpi()
-        if fig is not None:
-            plt.close(fig)
-        fig, ax = plt.subplots(1, 1)
-        fig = plt.gcf()
-        fig.set_size_inches(self.canvas_x / float(dpi), self.canvas_y / float(dpi))
-        fig.set_tight_layout(True)
-        plt.margins(x=0)
+                          plot_hkls: dict, plot_norm_int: dict,
+                          axis_scale: dict, fontdict: dict,
+                          fig: Figure) -> Figure:
+
+        if fig:
+            fig.clear()
+        fig = Figure(figsize=(6, 3), dpi=self.dpi)
+        ax = fig.add_subplot()
+        fig.set_tight_layout(True)  # https://github.com/matplotlib/matplotlib/issues/21970 https://github.com/matplotlib/matplotlib/issues/11059
+        ax.margins(x=0)
 
         # xs and ys hold unscaled values
         xs, ys, plot_list = self.get_all_xy_data(x_ordinate, y_ordinate)
         offset = _scale_y_ordinate(offset, axis_scale)
 
         if plot_norm_int["norm_int"]:
-            print(f'{plot_norm_int["y_ordinate_for_norm"]=}')
             y_norms = self.get_xy_ynorm_data(x_ordinate, plot_norm_int["y_ordinate_for_norm"])
         else:
             y_norms = [np.ones(len(xi)) for xi in xs]
 
         # compile all the patterns' data
         # need to loop backwards so that the data comes out in the correct order for plotting
-
+        hover_texts = []
         for j in range(len(plot_list) - 1, -1, -1):
             pattern = plot_list[j]
-            x, y = xs[j], ys[j]*y_norms[j]
+            x, y = xs[j], ys[j] * y_norms[j]
             x, y = _scale_xy_ordinates(x, y, axis_scale)
             label = pattern if not plot_norm_int["norm_int"] else f"{pattern} (norm.)"
-            ax.plot(x, y + j * offset, label=label)  # do I want to fill white behind each plot?
-
+            ax.plot(x, y + j * offset, label=label)
+            label = make_subtitle_string(self.cif[pattern], label)
+            hover_texts.append(label)
 
         # https://mplcursors.readthedocs.io/en/stable/examples/artist_labels.html
         stack_artists = ax.get_children()
-        mplcursors.cursor(stack_artists, hover=mplcursors.HoverMode.Transient).connect("add", lambda sel: sel.annotation.set_text(sel.artist.get_label()))
+        mplcursors.cursor(stack_artists,
+                          hover=mplcursors.HoverMode.Transient).connect("add", lambda sel: sel.annotation.set_text(hover_texts[stack_artists.index(sel.artist)]))
 
         if plot_hkls["above"] or plot_hkls["below"]:
             stack_hovertexts = []
@@ -522,20 +766,20 @@ class PlotCIF:
             for j in range(len(plot_list) - 1, -1, -1):
                 cifpat = self.cif[plot_list[j]]
                 hkl_y_offset = j * offset
-                x, y = xs[j], ys[j]*y_norms[j]
+                x, y = xs[j], ys[j] * y_norms[j]
                 x, y = _scale_xy_ordinates(x, y, axis_scale)
                 if "str" not in cifpat:
                     continue
-                tmp_hovertexts, tmp_hkl_artists = self.plot_hkls(plot_hkls["below"], cifpat, x_ordinate, x, [y, None],\
-                               axis_scale, 0, 0, hkl_y_offset, False, dpi, -1, ax)
+                tmp_hovertexts, tmp_hkl_artists = self.plot_hkls(plot_hkls["below"], cifpat, x_ordinate, x, [y, None],
+                                                                 axis_scale, 0, 0, hkl_y_offset, False, self.dpi, -1, ax)
                 stack_hovertexts.extend(tmp_hovertexts)
                 stack_hkl_artists.extend(tmp_hkl_artists)
 
-            self.add_hovertext_to_hkls(stack_hkl_artists, stack_hovertexts)
+            add_hovertext_to_each_point(stack_hkl_artists, stack_hovertexts)
         # end hkl if
 
         if x_ordinate in {"d", "_pd_proc_d_spacing"}:
-            plt.gca().invert_xaxis()
+            ax.invert_xaxis()
 
         if plot_norm_int["norm_int"]:
             y_axis_title = "Normalised counts"
@@ -552,31 +796,38 @@ class PlotCIF:
                 break
 
         x_axis_title, y_axis_title = _scale_xy_title(_x_axis_title(x_ordinate, wavelength), y_axis_title, axis_scale)
-        ax.set_xlabel(x_axis_title)
-        ax.set_ylabel(y_axis_title)
+        ax.set_xlabel(x_axis_title, fontdict=fontdict["xlabel"])
+        ax.set_ylabel(y_axis_title, fontdict=fontdict["ylabel"])
+
+        for label in ax.get_xticklabels():
+            label.update(fontdict["xticklabel"])
+        for label in ax.get_yticklabels():
+            label.update(fontdict["yticklabel"])
 
         return fig
 
     def surface_update_plot(self,
                             x_ordinate: str, y_ordinate: str, z_ordinate: str,
-                            plot_hkls: bool, plot_norm_int:dict,
-                            axis_scale: dict,
-                            fig: mf.Figure) -> mf.Figure:
-        dpi = plt.gcf().get_dpi()
-
-        if fig is not None:
-            plt.close(fig)
-        fig, ax = plt.subplots(1, 1)
-        fig = plt.gcf()
-        fig.set_size_inches(self.canvas_x / float(dpi), self.canvas_y / float(dpi))
-        fig.set_tight_layout(True)
-        plt.margins(x=0)
+                            plot_hkls: bool, plot_norm_int: dict, plot_metadata: str,
+                            axis_scale: dict, fontdict: dict,
+                            fig: Figure) -> Figure:
+        if fig:
+            fig.clear()
+        fig = Figure(figsize=(6, 3), dpi=self.dpi)
+        if plot_metadata:
+            ax, ax1 = fig.subplots(1, 2, gridspec_kw={'width_ratios': [2, 1]}, sharey=True)
+        else:
+            ax = fig.subplots(1, 1)
+        fig.set_tight_layout(True)  # https://github.com/matplotlib/matplotlib/issues/21970 https://github.com/matplotlib/matplotlib/issues/11059
+        ax.margins(x=0)
 
         # am I plotting the data I already have? If all the ordinates are the same, then I don't need to regrab all of the data
         #  and I can just use what I already have.
-        if self.surface_plot_data["x_ordinate"] == x_ordinate and \
-            self.surface_plot_data["y_ordinate"] == y_ordinate and \
-            self.surface_plot_data["z_ordinate"] == z_ordinate:
+        if (
+            self.surface_plot_data["x_ordinate"] == x_ordinate and
+            self.surface_plot_data["y_ordinate"] == y_ordinate and
+            self.surface_plot_data["z_ordinate"] == z_ordinate
+        ):
             debug("reusing surface plot data")
             xx = self.surface_plot_data["x_data"]
             yy = self.surface_plot_data["y_data"]
@@ -597,16 +848,15 @@ class PlotCIF:
         # end of if
 
         if plot_norm_int["norm_int"]:
-            xx, yy, zz = _scale_xyz_ordinates(xx, yy, zz*znorm, axis_scale)
+            xx, yy, zz = _scale_xyz_ordinates(xx, yy, zz * znorm, axis_scale)
         else:
             xx, yy, zz = _scale_xyz_ordinates(xx, yy, zz, axis_scale)
 
-        plt.pcolormesh(xx, yy, zz, shading='nearest', cmap=self.surface_z_color)
+        pcm = ax.pcolormesh(xx, yy, zz, shading='nearest', cmap=self.surface_z_color)
 
         if x_ordinate in {"d", "_pd_proc_d_spacing"}:
-            plt.gca().invert_xaxis()
+            ax.invert_xaxis()
 
-        # hkl plotting below     single_height_px
         surface_hovertexts = []
         surface_hkl_artists = []
         if plot_hkls:
@@ -657,8 +907,59 @@ class PlotCIF:
             z_axis_title = "Counts"
         x_axis_title, y_axis_title, z_axis_title = _scale_xyz_title(_x_axis_title(x_ordinate, wavelength), "Pattern number", z_axis_title, axis_scale)
 
-        plt.xlabel(x_axis_title)
-        plt.ylabel(y_axis_title)
-        plt.colorbar(label=z_axis_title)
+        ax.set_xlabel(x_axis_title, fontdict=fontdict["xlabel"])
+        ax.set_ylabel(y_axis_title, fontdict=fontdict["ylabel"])
 
+        for label in ax.get_xticklabels():
+            label.update(fontdict["xticklabel"])
+        for label in ax.get_yticklabels():
+            label.update(fontdict["yticklabel"])
+
+        # make second subplot here
+        if plot_metadata:  # string == "temp", "pres", "qpa"
+            y = [_scale_y_ordinate(i + 1, axis_scale) for i in range(len(self.cif))]
+            if plot_metadata != "qpa":
+                second_plot = []
+                for pattern in self.cif:
+                    cifpat = self.cif[pattern]
+                    if plot_metadata == "temp":
+                        second_data = get_dict_value(cifpat, "_diffrn_ambient_temperature")
+                        ax1_label = "Temperature (K)"
+                    elif plot_metadata == "pres":
+                        second_data = get_dict_value(cifpat, "_diffrn_ambient_pressure")
+                        ax1_label = "Pressure (kPa)"
+                    elif plot_metadata == "rwp":
+                        second_data = get_dict_value(cifpat, "_pd_proc_ls_prof_wr_factor") * 100
+                        ax1_label = "Rwp (%)"
+                    elif plot_metadata == "gof":
+                        second_data = get_dict_value(cifpat, "_refine_ls_goodness_of_fit_all")
+                        ax1_label = "GoF"
+
+                    second_plot.append(second_data)
+                    ax1.set_xlabel(ax1_label, fontdict=fontdict["xlabel"])
+                    for label in ax1.get_xticklabels():
+                        label.update(fontdict["xticklabel"])
+
+                ax1.plot(second_plot, y, marker="o")
+            else:
+                amor_qpa = 100 * np.ones(len(y))
+                for phase in self.qpa:
+                    ax1.plot(self.qpa[phase], y, label=phase)
+                    amor_qpa = np.subtract(amor_qpa, np.nan_to_num(self.qpa[phase]))
+                if np.any(amor_qpa > 0.5):
+                    ax1.plot(amor_qpa, y, label="Unknown", color="black", linestyle='dashed')
+                surface_qpa_artists = ax1.get_children()
+                mplcursors.cursor(surface_qpa_artists, hover=mplcursors.HoverMode.Transient).connect("add", lambda sel: sel.annotation.set_text(sel.artist.get_label()))
+                ax1.set_xlabel("Weight percent (wt%)", fontdict=fontdict["xlabel"])
+                ax1.legend(prop=fontdict["legend"], labelcolor=fontdict["legendcolor"]["color"])
+                for label in ax1.get_xticklabels():
+                    label.update(fontdict["xticklabel"])
+            ax1.grid(True, color='lightgrey')
+            cb = fig.colorbar(pcm, ax=ax1)  # this is the code that adds the colour bar.
+        else:
+            cb = fig.colorbar(pcm, ax=ax)
+
+        cb.set_label(z_axis_title, **fontdict["zlabel"])
+        for label in cb.ax.get_yticklabels():
+            label.update(fontdict["zticklabel"])
         return fig
